/*****************************************************************************
 * Source code information
 * -----------------------
 * Original author    Ian Dickinson, HP Labs Bristol
 * Author email       Ian_Dickinson@hp.com
 * Package            Jena performanace tests
 * Created            11 June 2002
 * Filename           $RCSfile: AbstractPerfTest.java,v $
 * Revision           $Revision: 1.3 $
 * Release status     Experimental. Do not distribute. $State: Exp $
 *
 * Last modified on   $Date: 2003/05/02 07:44:06 $
 *               by   $Author: jeremy_carroll $
 *
 * @copyright@
 *****************************************************************************/

// Package
///////////////
package com.hp.hpl.jena.test.performance;

import java.text.SimpleDateFormat;
import java.util.*;

// Imports
///////////////

/**
 * <p>
 * Abstract base class for simple Jena performance benchmarking.
 * </p>
 *
 * @author Ian Dickinson, HP Labs (<a href="mailto:Ian.Dickinson@hp.com">email</a>)
 * @version CVS info: $Id: AbstractPerfTest.java,v 1.3 2003/05/02 07:44:06 jeremy_carroll Exp $
 */
public abstract class AbstractPerfTest {
	// Constants
	//////////////////////////////////

	public static final String DEFAULT_FIELD_SEP = ",";

	public static final int DEFAULT_SIZE = 10;

	public static final int DEFAULT_COUNT = 5;

	public static final long MEASUREMENT_THRESHOLD = 250L;

	// Static variables
	//////////////////////////////////
	

	// Instance variables
	//////////////////////////////////

	// Constructors
	//////////////////////////////////

	// External signature methods
	//////////////////////////////////

	/**
	 * Write to standard out the environment conditions for this test and the result
	 * 
	 * @param sep The field separator, default = ","
	 * @param size Context-dependent size parameter, e.g. the number of triples parsed in this run
	 * @param time The average time taken, in milliseconds
	 */
	public void writeResult(String sep, int size, double time) {
		String s = sep == null ? DEFAULT_FIELD_SEP : sep;

		StringBuffer buf = new StringBuffer();

		buf.append("Jena2-PerfTest");
		buf.append(s);
		//buf.append( new SimpleDateFormat( "yyyy-M-dd HH:mm:ss z" ).format( new Date() ) );
		//buf.append( s );
		buf.append(System.getProperty("java.vm.name"));
		buf.append(" ");
		buf.append(System.getProperty("java.vm.version"));
		buf.append(" ");
		buf.append(System.getProperty("os.name"));
		buf.append(" ");
		buf.append(System.getProperty("os.arch"));
		buf.append(s);
		buf.append(getDescription());
		buf.append(s);
		buf.append(size);
		buf.append(s);
		buf.append(time);

		System.out.println(buf.toString());
	}

	/**
	 * Standard entry point - run the test, and report the result. Use
	 * <code><pre>
	 *    -Djena.perftest.size=NNN
	 * </pre></code>
	 * to set the size of the test to run. Use
	 * <code><pre>
	 *    -Djena.perftest.fieldsep=s
	 * </pre></code>
	 * to set the output field separator. Use
	 * <code><pre>
	 *    -Djena.perftest.count=MMM
	 * </pre></code>
	 * to set the iteration count.
	 */
	protected void go(String[] args) {
		int size =
			Integer.getInteger("jena.perftest.size", DEFAULT_SIZE).intValue();
		String sep =
			System.getProperty("java.perftest.fieldsep", DEFAULT_FIELD_SEP);
		int count =
			Integer.getInteger("jena.perftest.count", DEFAULT_COUNT).intValue();

		long duration = 0L;
		boolean acceptableResult = false;
		
		try {
			setupTest(size);

			for (int i = 0; i < 2; i++) {
// i = 0 is a dry run that is discarded.
// i = 1 we record results.
                acceptableResult = false;
				while (!acceptableResult) {
					long start = System.currentTimeMillis();
					for (int j = 0; j < count; j++) {
						runTest(size);
						System.gc();
					}
					duration = System.currentTimeMillis() - start;

					// check that the duration was long enough to give a reliable result
					if (duration < MEASUREMENT_THRESHOLD) {
						count = count * 2;
					} else {
						acceptableResult = true;
					}
				}
			}

			endTest();
		} catch (Exception e) {
			System.err.println(
				"Test "
					+ getDescription()
					+ " failed with "
					+ e.getClass().getName());
			e.printStackTrace(System.err);
		}

		writeResult(sep, size, (double) duration / (double) count);
	}

	/**
	 * Abstract method that runs the actual test.
	 * 
	 * @param size Context dependent size parameter, such as the number of statements to create
	 */
	public abstract void runTest(int size) throws Exception;

	/**
	 * Answer the human readable description of the test, that will appear on the generated html
	 * @return The description of the test <b>which should not contain any commas</b>
	 */
	public abstract String getDescription();

	/**
	 * Do any necessary setup for the main test. Default is to do nothing.
	 * 
	 * @param size The (context dependent) size parameter for this test
	 */
	protected void setupTest(int size) throws Exception {
	}

	/**
	 * Clean up at the end of the test. Default is to do nothing.
	 */
	protected void endTest() throws Exception {
	}

	// Internal implementation methods
	//////////////////////////////////

	//==============================================================================
	// Inner class definitions
	//==============================================================================

}

/* 
@footer@ 
*/
